
#ifndef _CEF_SCHEME_H
#define _CEF_SCHEME_H

#include <string>
#include "include/cef.h"
#include "include/cef_ptr.h"
#include "include/cef_types.h"

class CefSchemeHandler;
class CefSchemeHandlerFactory;

// Register a custom scheme handler using the CefSchemeHandler interface
// Custom scheme in the format of scheme_name://host_name/ will be handled through
// the specified handler interface.
// All URLs beginning with scheme_name://host_name/ will go through this handler
/*--cef()--*/
bool CefRegisterScheme(	const std::wstring& scheme_name,
												const std::wstring& host_name,
												CefRefPtr<CefSchemeHandlerFactory> factory);

// Class that creates CefSchemeHandler instances.
/*--cef(source=client)--*/
class CefSchemeHandlerFactory : public CefBase
{
public:
	// Return a new scheme handler instance to handle the request.
	/*--cef()--*/
	virtual CefRefPtr<CefSchemeHandler> Create() =0;
};

// Class used to represent a custom scheme handler interface.
/*--cef(source=client)--*/
class CefSchemeHandler : public CefBase
{
public:
	// Process the request. All response generation should take place in this
	// method.
	/*--cef()--*/
	virtual bool ProcessRequest(CefRefPtr<CefRequest> request) =0;

	// Cancel processing of the request.
	/*--cef()--*/
	virtual void Cancel() =0;

	// Return the mime type for the request.
	/*--cef()--*/
	virtual std::wstring GetMimeType() =0;

	// Return the response length for the request. If there is no response return
	// 0 and ReadResponse() will not be called. If the response length is not
	// known return -1 and ReadResponse() will be called until it returns false or
	// until the value of |bytes_read| is set to 0. If the response length is
	// known return a positive value and ReadResponse() will be called until it
	// returns false, the value of |bytes_read| is set to 0 or the specified
	// number of bytes have been read.
	/*--cef()--*/
	virtual size_t GetResponseLength() =0;

	// Copy up to |bytes_to_read| bytes into |data_out|. If the copy succeeds
	// set |bytes_read| to the number of bytes copied and return true. If the
	// copy fails return false and ReadResponse() will not be called again.
	/*--cef()--*/
	virtual bool ReadResponse(void* data_out, int bytes_to_read,
		int* bytes_read) =0;
};

#endif