// Copyright (c) 2009 Marshall A. Greenblatt. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//    * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//    * Neither the name of Google Inc. nor the name Chromium Embedded
// Framework nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// ---------------------------------------------------------------------------
//
// This file was generated by the CEF translator tool and should not edited
// by hand. See the translator.README.txt file in the tools directory for
// more information.
//

#ifndef _CEF_SCHEME_CAPI_H
#define _CEF_SCHEME_CAPI_H

#ifdef __cplusplus
extern "C" {
#endif

#include "cef_capi.h"
#include "cef_export.h"
#include "cef_string.h"
#include "cef_string_list.h"
#include "cef_string_map.h"
#include "cef_types.h"


// Register a custom scheme handler using the cef_scheme_handler_t structure
// Custom scheme in the format of scheme_name://host_name/ will be handled
// through the specified handler structure. All URLs beginning with
// scheme_name://host_name/ will go through this handler
CEF_EXPORT int cef_register_scheme(const wchar_t* scheme_name,
    const wchar_t* host_name, struct _cef_scheme_handler_factory_t* factory);

// Check that the structure |s|, which is defined with a cef_base_t member named
// |base|, is large enough to contain the specified member |f|.
#define CEF_MEMBER_EXISTS(s, f)   \
  ((int)&((s)->f) - (int)(s) + sizeof((s)->f) <= (s)->base.size)

#define CEF_MEMBER_MISSING(s, f)  (!CEF_MEMBER_EXISTS(s, f) || !((s)->f))


// Structure that creates cef_scheme_handler_t instances.
typedef struct _cef_scheme_handler_factory_t
{
  // Base structure.
  cef_base_t base;

  // Return a new scheme handler instance to handle the request.
  struct _cef_scheme_handler_t* (CEF_CALLBACK *create)(
      struct _cef_scheme_handler_factory_t* self);

} cef_scheme_handler_factory_t;


// Structure used to represent a custom scheme handler structure.
typedef struct _cef_scheme_handler_t
{
  // Base structure.
  cef_base_t base;

  // Process the request. All response generation should take place in this
  // function.
  int (CEF_CALLBACK *process_request)(struct _cef_scheme_handler_t* self,
      struct _cef_request_t* request);

  // Cancel processing of the request.
  void (CEF_CALLBACK *cancel)(struct _cef_scheme_handler_t* self);

  // Return the mime type for the request.
  // The resulting string must be freed by calling cef_string_free().
  cef_string_t (CEF_CALLBACK *get_mime_type)(
      struct _cef_scheme_handler_t* self);

  // Return the response length for the request. If there is no response return
  // 0 and read_response() will not be called. If the response length is not
  // known return -1 and read_response() will be called until it returns false
  // (0) or until the value of |bytes_read| is set to 0. If the response length
  // is known return a positive value and read_response() will be called until
  // it returns false (0), the value of |bytes_read| is set to 0 or the
  // specified number of bytes have been read.
  size_t (CEF_CALLBACK *get_response_length)(
      struct _cef_scheme_handler_t* self);

  // Copy up to |bytes_to_read| bytes into |data_out|. If the copy succeeds set
  // |bytes_read| to the number of bytes copied and return true (1). If the copy
  // fails return false (0) and read_response() will not be called again.
  int (CEF_CALLBACK *read_response)(struct _cef_scheme_handler_t* self,
      void* data_out, int bytes_to_read, int* bytes_read);

} cef_scheme_handler_t;


#ifdef __cplusplus
}
#endif

#endif // _CEF_SCHEME_CAPI_H
