
#include "cef.h"
#include "../libcef/scheme/cef_scheme.h"
#include "third_party/skia/include/core/SkCanvas.h"
#include "third_party/skia/include/core/SkBitmap.h"
#include "third_party/skia/include/effects/SkGradientShader.h"
#include "third_party/skia/include/effects/SkBlurDrawLooper.h"
#include "skia/ext/platform_canvas.h"
#include "skia/ext/bitmap_platform_device.h"
#include "base/gfx/png_encoder.h"
#include "base/string_util.h"

using namespace skia;
long g_count = 0;

class CustomSchemeHandler : public CefThreadSafeBase<CefSchemeHandler>
{
public:
	virtual bool ProcessRequest(CefRefPtr<CefRequest> request)
	{
		PrepareContent(request);
		return true;
	}
	virtual void Cancel()
	{
	}
	virtual size_t GetResponseLength()
	{
		return buffer_.size();
	}
	virtual bool ReadResponse(void* data_out, int bytes_to_read, int *bytes_read)
	{
		if (buffer_.size()>0)
		{
			int bytes_to_copy = static_cast<int>(
				std::min(static_cast<long long>(bytes_to_read), 
				static_cast<long long>(buffer_.size() - buffer_pos_)));
			if (bytes_to_copy>0)
			{
				memcpy(data_out, &buffer_[buffer_pos_], bytes_to_copy);
				*bytes_read = bytes_to_copy;
				buffer_pos_ += bytes_to_copy;
				return true;
			}
			else
			{
				*bytes_read = 0;
				return false;
			}
		}
		else
		{
			*bytes_read = 0;
			return false;
		}
	}
	virtual std::wstring GetMimeType()
	{
		return mimetype_;
	}

private:

	void PrepareContent(CefRefPtr<CefRequest>& request)
	{
		buffer_.resize(0);
		buffer_pos_ = 0;
		if (wcsstr(request->GetURL().c_str(), L"custom://host/image") )
		{
			_SampleImageGenerator(request);
			return;
		}
		else if(request->GetURL().compare(L"custom://host/xml") == 0)
		{
			_SampleXmlPostHandler(request);
			return;
		}
		std::string s;
		_PrintCommonHeader(s);
		_PrintScriptBlock(s);

		s+="<div>CustomSchemeHandler::PrepareContent, serving: ";
		s+=WideToUTF8(request->GetURL());
		s+="</div>";

		s+="<h1>Image Test</h1>";
		s+="<div>You should see 9 programmatically generated image below by this scheme handler.</div>";
		s+="<div><img src='custom://host/image1'></img>";
 		s+="<img src='custom://host/image2'></img>";
		s+="<img src='custom://host/image3'></img>";
		s+="<img src='custom://host/image4'></img>";
		s+="<img src='custom://host/image5'></img>";
		s+="<img src='custom://host/image6'></img>";
		s+="<img src='custom://host/image7'></img>";
		s+="<img src='custom://host/image8'></img>";
		s+="<img src='custom://host/image9'></img></div>";
		s+="<div>* Note: These images are generated separately and concurrently, indicating successful simultaneous request processing.</div>";
		s+="<div>* Note: The counter in image will add up when you refresh this page.</div>";

		s+="<h1>XMLHttpRequest Test</h1>";
		s+="<div>* Note: This will not work if compiled using the official chromium source tree.</div>";
		s+="<div>* Something should appear between the following lines when you pressed the buttons below.</div>";
		s+="<hr/><div id=\"cmdresponse\" style=\"color: red; font-weight: bold\"></div><hr/>";
		s+="<input type=\"button\" value=\"Send GetDate Command\" onclick=\"xmlpost('getdate');\"/> ";
		s+="<input type=\"button\" value=\"Send Unknown Command\" onclick=\"xmlpost('unknown');\"/> ";
		s+="<br/>";

		s+="<h1>Post Test</h1>";
		s+="<div>Post Data:<br/>";
		s+="# elements: ";
		char szCount[65];
		long nCount = 0;
		if(request->GetPostData())
			nCount = request->GetPostData()->GetElementCount();
		itoa(nCount, szCount, 10);
		s+="<br/>";
		if (nCount>0)
		{
			CefPostData::ElementVector elems;
			request->GetPostData()->GetElements(elems);

			CefRefPtr<CefPostDataElement> element;
			CefPostData::ElementVector::const_iterator it = elems.begin();
			for(; it != elems.end(); ++it)
			{
				element = (*it);
				if(element->GetType() == PDE_TYPE_BYTES && element->GetBytesCount()>0)
				{
					std::vector<char> elem_data;
					elem_data.resize(element->GetBytesCount());
					element->GetBytes(elem_data.size(), &elem_data[0]);
					s+="<pre>Post Data: ";
					s+=std::string(&elem_data[0], elem_data.size());
					s+="\n";
					s+="</pre>";
				}
			}
		}
		s+="</div>";
		s+="<div>Sample Post Form:";
		s+="<form name='myform' action=\"/post\" method=\"post\">";
		s+="<input type='text' name='keywords'/>";
		s+="<input type='submit' name='submit'/><br/>";
		s+="</form>";
		s+="</div>";

		_PrintCommonFooter(s);

		buffer_.resize(s.length());
		memcpy(&buffer_[0], s.c_str(), s.length());

		mimetype_=L"text/html";
	}

	void _SampleXmlPostHandler( CefRefPtr<CefRequest>& request )
	{
		std::string s = "<node>Unknown Command</node>";
		long nCount = 0;
		if(request->GetPostData())
			nCount = request->GetPostData()->GetElementCount();
		if (nCount>0)
		{
			CefPostData::ElementVector elems;
			request->GetPostData()->GetElements(elems);

			CefRefPtr<CefPostDataElement> element;
			CefPostData::ElementVector::const_iterator it = elems.begin();
			for(; it != elems.end(); ++it)
			{
				element = (*it);
				if(element->GetType() == PDE_TYPE_BYTES && element->GetBytesCount()>0)
				{
					std::vector<char> elem_data;
					elem_data.resize(element->GetBytesCount());
					element->GetBytes(elem_data.size(), &elem_data[0]);
					if(std::string(&elem_data[0], elem_data.size()).compare("<node>getdate</node>") == 0)
					{
						SYSTEMTIME st;
						GetSystemTime(&st);
						char cdate[200];
						GetDateFormatA(LOCALE_USER_DEFAULT , DATE_LONGDATE, &st, NULL, cdate, 200);
						s = "<node>";
						s += cdate;
						s += "</node>";
					}
					break;
				}
			}
		}
		buffer_.resize(s.length());
		memcpy(&buffer_[0], s.c_str(), s.length());
		mimetype_=L"text/xml";
	}

	void _SampleImageGenerator( CefRefPtr<CefRequest>& request )
	{
		g_count++;
		unsigned int width = 60, height = 50;

		SkPaint pa1, pa2, pa3;

		SkPoint grad_points[2];
		grad_points[0].set(SkIntToScalar(0), SkIntToScalar(0));
		grad_points[1].set(SkIntToScalar(0), SkIntToScalar(height));

		int uClrR1 = (double)rand() / (RAND_MAX + 1) * (0xFF-0xBB)+0xBB;
		int uClrG1 = (double)rand() / (RAND_MAX + 1) * 0xBB;
		int uClrB1 = (double)rand() / (RAND_MAX + 1) * 0xBB;
		int uClrR2 = (double)rand() / (RAND_MAX + 1) * (0xFF-0xBB)+0xBB;
		int uClrG2 = (double)rand() / (RAND_MAX + 1) * 0xBB;
		int uClrB2 = (double)rand() / (RAND_MAX + 1) * 0xBB;

		SkColor grad_colors[2] = { 0xFF000000 + (uClrR1 << 16) + (uClrG1 << 8) + uClrB1, 0xFF000000 + (uClrR2 << 16) + (uClrG2 << 8) + uClrB2};

		SkBlurDrawLooper* l = new  SkBlurDrawLooper(
			2,
			0,
			0,
			0xA0000000);
		pa1.setLooper(l);
		l->unref();

		pa1.setColor(0xFFFFFFFF);
		pa1.setStyle(SkPaint::kFill_Style);
		pa1.setAntiAlias(true);
		pa1.setStrokeWidth(1);

		pa3.setAntiAlias(true);
		pa3.setColor(0xFFFFFFFF);
		pa3.setTextSize(10);
		pa3.setLCDRenderText(true);

		SkBitmap bitmap;
		bitmap.setConfig(SkBitmap::kARGB_8888_Config, width, height);
		bitmap.setIsOpaque(false);
		bitmap.allocPixels();
		bitmap.eraseColor(0x00000000);
		SkCanvas canvas(bitmap);

		SkRect rr = {3, 3, width-6, height-6};
		canvas.drawRoundRect(rr, 5, 5, pa1);

		SkShader* shader = SkGradientShader::CreateLinear(grad_points, grad_colors, NULL, 2, SkShader::kClamp_TileMode);
		pa1.setShader(shader);
		shader->unref();
		pa1.setLooper(NULL);

		canvas.drawRoundRect(rr, 5, 5, pa1);

		char num[65];
		itoa(g_count, num, 10);
		canvas.drawText(num, strlen(num), 8, 20, pa3);

		PNGEncoder::EncodeBGRASkBitmap(bitmap, false, &buffer_);

		mimetype_ = L"image/png";
	}

	void _PrintCommonHeader(std::string& s)
	{
		s+="<head>"
			"<title>Custom Scheme Handler</title>"
			"<style type=\"text/css\">"
			"body, input, td, th, h1, div {"
			"font-family: \"Segoe UI\", Tahoma;"
			"font-size: 12px;"
			"}"
			"h1 {"
			"font-size: 18px;"
			"font-weight: normal;"
			"}"
			"</style>"
			"</head>"
			"<body>";
	}

	void _PrintScriptBlock(std::string& s)
	{
		s+="<script language='javascript'>"
			"function xmlpost(cmd){"
			"var xhr = new XMLHttpRequest();"
			"xhr.open(\"POST\", \"custom://host/xml\", false);"
			"xhr.send(\"<node>\"+cmd+\"</node>\");"
			"cmdresponse.innerHTML = xhr.responseText;"
			"}"
			"</script>";

	}

	void _PrintCommonFooter(std::string& s)
	{
		s+="</body></html>";
	}

	std::vector<unsigned char> buffer_;
	std::wstring mimetype_;
	size_t buffer_pos_;

};


class CustomSchemeHandlerFactory : public CefThreadSafeBase<CefSchemeHandlerFactory>
{
public:
	virtual CefRefPtr<CefSchemeHandler> Create()
	{
		return new CustomSchemeHandler();
	}
};

